﻿using System;
using System.Linq;
using log4net;
using log4net.Appender;
using log4net.Repository.Hierarchy;
using NHibernate.Linq;
using NUnit.Framework;

namespace NHibernate.Test.NHSpecificTest.NH2439 {
    [TestFixture]
    public class Test : BugTestCase {
        //private MemoryAppender memoryAppender;


        protected override void OnSetUp()
        {
            using (ISession session = OpenSession()) {
                Organisation organisation = new Organisation();
                TrainingComponent trainingComponent = new TrainingComponent() {Code = "123", Title = "title"};
                RtoScope scope = new RtoScope();
                scope.Nrt = trainingComponent;
                scope.Rto = organisation;
                scope.StartDate = DateTime.Today.AddDays(-100);

                session.Save(organisation);
                session.Save(trainingComponent);
                session.Save(scope);
                
                OrganisationSearchResult searchResult = new OrganisationSearchResult() {Organisation = organisation};
                session.Save(searchResult);
                session.Flush();

            }
            base.OnSetUp();
        }
        [Test]
        public void TheTest() {
            using (ISession session = OpenSession()) {
                IQueryable<RtoScope> scopes= session.Query<RtoScope>().Where(s => s.StartDate<=DateTime.Today && (s.EndDate==null || s.EndDate>=DateTime.Today) && !s.IsRefused );
                string filter = "ABC";
                scopes = scopes.Where(t => t.Nrt.Title.Contains(filter) || t.Nrt.Code == filter);

                IQueryable<OrganisationSearchResult> query = session.Query<OrganisationSearchResult>();
                var finalQuery = query.Where(r=>scopes.Any(s=>s.Rto==r.Organisation));
                IQueryable<Organisation> organisations= scopes.Select(s => s.Rto);

                Assert.That(scopes.ToList().Count==0);
                Assert.That(organisations.ToList().Count==0);
                //the SQL in below fails - the organisations part of the query is not included at all..
                Assert.That(query.Where(r => organisations.Contains(r.Organisation)).ToList().Count()==0);
                var list = finalQuery.ToList();
                Assert.That(list.Count==0);
            }
        }
        protected override void OnTearDown()
        {
            using (ISession session = OpenSession()) {
                session.Delete("from OrganisationSearchResult");
                session.Delete("from RtoScope");
                session.Delete("from Organisation");
                session.Delete("from TrainingComponent");
                session.Flush();
            }
            
        }
    }
}